package com.example.comicfinder;

import android.app.Activity;
import android.graphics.Canvas;
import android.graphics.ImageFormat;
import android.hardware.Camera;
import android.hardware.Camera.Size;
import android.os.Bundle;
import android.view.SurfaceView;
import android.view.Window;
import android.view.WindowManager;

public class MainActivity extends Activity {
	private Camera camera;
	private Size previewSize;
	private int[] previewBufferRgba;

	private SurfaceView surface;

	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);

		// setContentView()より先に呼ぶこと。
		getWindow().addFlags(WindowManager.LayoutParams.FLAG_FULLSCREEN);
		requestWindowFeature(Window.FEATURE_NO_TITLE);

		setContentView(R.layout.activity_main);

		surface = (SurfaceView)findViewById(R.id.surface);
	}

	@Override
	protected void onResume() {
		super.onResume();

		camera = Camera.open();
		if (camera == null) {
			finish();
			return;
		}

		Camera.Parameters params = camera.getParameters();

		// プレビューフォーマットをNV21に設定
		if (ImageFormat.NV21 != params.getPreviewFormat()) {
			if (params.getSupportedPreviewFormats().contains(ImageFormat.NV21)) {
				finish();
				return;
			}

			params.setPreviewFormat(ImageFormat.NV21);
		}

		// プレビューサイズを(横幅が)最も大きいものに設定
		for (Size s : params.getSupportedPreviewSizes()) {
			if (params.getPreviewSize().width < s.width) {
				params.setPreviewSize(s.width, s.height);
			}
		}

		camera.setParameters(params);
		previewSize = params.getPreviewSize();

		// プレビューのrawデータ。8bit * (画サイズ) * 1.5
		byte[] previewBuffer = new byte[previewSize.width * previewSize.height * 3 / 2];
		// 描画用バッファ。32bit * (画サイズ)
		previewBufferRgba = new int[previewSize.width * previewSize.height];

		camera.setPreviewCallbackWithBuffer(new Camera.PreviewCallback() {
			@Override
			public void onPreviewFrame(byte[] data, Camera camera) {
				updateFrame(data);
			}
		});
		camera.startPreview();
		camera.addCallbackBuffer(previewBuffer);
	}

	@Override
	protected void onPause() {
		super.onPause();

		if (camera != null) {
			camera.stopPreview();
			camera.release();
		}
	}

	private void updateFrame(byte[] data) {
		int lum;
		for (int k = 0; k < previewSize.width * previewSize.height; k++) {
			lum = data[k] & 0xff;
			previewBufferRgba[k] = (0xff << 24) | (lum << 16) | (lum << 8) | lum;
		}

		Canvas c = surface.getHolder().lockCanvas();
		if (c != null) {
			try {
				// 表示域をはみ出す部分は切り捨てる。また、センタリングしない。
				float scale = Math.max((float)surface.getWidth() / previewSize.width, (float)surface.getHeight() / previewSize.height);
				c.scale(scale, scale);
				c.drawBitmap(previewBufferRgba, 0, previewSize.width, 0, 0, previewSize.width, previewSize.height, true, null);
			}
			finally {
				surface.getHolder().unlockCanvasAndPost(c);
			}
		}

		camera.addCallbackBuffer(data);
	}

}
